@@ -135,10 +135,10 @@ ac3iec_base_init (gpointer g_class)
       "Codec/Muxer/Audio",
       "Pads AC3 frames into IEC958 frames suitable for a raw S/PDIF interface",
       "Martin Soto <martinsoto@users.sourceforge.net>");
-  gst_element_class_add_static_pad_template (element_class,
-      &ac3iec_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &ac3iec_src_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&ac3iec_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&ac3iec_src_template));
 }
 
 
@@ -160,7 +160,7 @@ ac3iec_class_init (AC3IECClass * klass)
   g_object_class_install_property (gobject_class, PROP_RAW_AUDIO,
       g_param_spec_boolean ("raw-audio", "raw-audio",
           "If true, source pad caps are set to raw audio.",
-          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          FALSE, G_PARAM_READWRITE));
 
   gstelement_class->change_state = ac3iec_change_state;
 }
